/* Handles Signal Interrupts and Conversions */

#ifndef SIGNALS_H_
#define SIGNALS_H_

void catch_signals();
void signal_handler(int sig);
void num_to_sig(int sig,FILE *stream);
int check_wait_status(int status,FILE *stream);
void fd_to_name(int fd);
int close_ioe();

/* cl->out_stream might not be initalized */
void signal_handler(int sig)
{
    signal(sig, SIG_IGN);   
    
    if (cl->out_stream==NULL)
    {
        fprintf(stderr,"\n*** Caught: ");
        num_to_sig(sig,stderr); fprintf(stderr," ***\n");
    }
    else
    {
        fprintf(cl->out_stream,"\n*** Caught: ");
        num_to_sig(sig,cl->out_stream); fprintf(cl->out_stream," ***\n");
    }
    
    if (cl->child_pid >= 0)
    {
        if (cl->out_stream==NULL)
            fprintf(stderr,"+++ %i Detaching from pid %i +++\n",getpid(),cl->child_pid);
        else
            fprintf(cl->out_stream,"+++ %i Detaching from pid %i +++\n",
                getpid(),cl->child_pid);
    
        /* Check the status of the pid one more time before detaching */
        /* We have to check if we did an -e or -p use one of these below */
        //waitpid(cl->child_pid, NULL, WUNTRACED);
        //waitpid(cl->child_pid, NULL, WNOHANG);
        /* ************************************* we need to distinguish from -p or -e here */
        waitpid(cl->child_pid, NULL, WNOHANG); // -e safe, causes wierd status in -p
        DETACH(cl->child_pid); //PKILL(cl->child_pid); // -e safe
    }
    
    exit(sig);
}

/* called by signal_handler(int sig) */
void num_to_sig(int sig,FILE *stream)
{
    switch(sig)
    {
       case SIGHUP: fprintf(stream,"SIGHUP"); break;
       case SIGINT: fprintf(stream,"SIGINT"); break;
       case SIGQUIT: fprintf(stream,"SIGQUIT"); break;
       case SIGILL: fprintf(stream,"SIGILL"); break;
       case SIGABRT: fprintf(stream,"SIGABRT"); break;
       case SIGFPE: fprintf(stream,"SIGFPE"); break;
       case SIGKILL: fprintf(stream,"SIGKILL"); break;
       case SIGSEGV: fprintf(stream,"SIGSEGV (Segmentation fault)"); break;
       case SIGPIPE: fprintf(stream,"SIGPIPE"); break;
       case SIGALRM: fprintf(stream,"SIGALRM"); break;
       case SIGTERM: fprintf(stream,"SIGTERM"); break;
       case SIGUSR1: fprintf(stream,"SIGUSR1"); break;
       case SIGUSR2: fprintf(stream,"SIGUSR2"); break;
       case SIGCHLD: fprintf(stream,"SIGCHLD"); break;
       case SIGCONT: fprintf(stream,"SIGCONT"); break;
       case SIGSTOP: fprintf(stream,"SIGSTOP"); break;
       case SIGTSTP: fprintf(stream,"SIGTSTP"); break;
       case SIGTTIN: fprintf(stream,"SIGTTIN"); break;
       case SIGTTOU: fprintf(stream,"SIGTTOU"); break;
       case SIGBUS: fprintf(stream,"SIGBUS"); break;
       case SIGPOLL: fprintf(stream,"SIGPOLL"); break;
       case SIGPROF: fprintf(stream,"SIGPROF"); break;
       case SIGSYS: fprintf(stream,"SIGSYS"); break;
       case SIGTRAP: fprintf(stream,"SIGTRAP"); break;
       case SIGURG: fprintf(stream,"SIGURG"); break;
       case SIGVTALRM: fprintf(stream,"SIGVTALRM"); break;
       case SIGXCPU: fprintf(stream,"SIGXCPU"); break;
       case SIGXFSZ: fprintf(stream,"SIGXFSZ"); break;
       /* Extended Signals: Not all are defined
       case SIGIOT: fprintf(stream,"SIGIOT"); break;
       case SIGEMT: fprintf(stream,"SIGEMT"); break;
       case SIGSTKFLT: fprintf(stream,"SIGSTKFLT"); break;
       case SIGIO: fprintf(stream,"SIGIO"); break;
       case SIGCLD: fprintf(stream,"SIGCLD"); break;
       case SIGPWR: fprintf(stream,"SIGPWR"); break;
       case SIGINFO: fprintf(stream,"SIGINFO"); break;
       case SIGLOST: fprintf(stream,"SIGLOST"); break;
       case SIGWINCH: fprintf(stream,"SIGWINCH"); break;
       case SIGUNUSED: fprintf(stream,"SIGUNUSED"); break;
       */
       default: fprintf(stream,"Unknown");
    }
    fprintf(stream,"[%i]",sig);
}


int check_wait_status(int status,FILE *stream)
{
     int s = 0;
     //fprintf(stream,"wait status %i: ",status);
     if (WIFEXITED(status))
     {
        s = WEXITSTATUS(status);
        /*fprintf(stream,"Child terminated: ");*/
        fprintf(stream,"+++ %i exited (status %i) +++",cl->child_pid,s);
     }
     else if (WIFSIGNALED(status))
     {
        fprintf(stream,"*** %i signal",cl->child_pid);
        s = WTERMSIG(status);
        num_to_sig(s,stream);
#ifdef WCOREDUMP
        if (WCOREDUMP(status))
            fprintf(stream,"(with core) ***");
        else
#endif
        fprintf(stream," ***");
     }
     else if (WIFSTOPPED(status))
     {
        fprintf(stream,"*** %i halted: ",cl->child_pid);
        s = WSTOPSIG(status);
        num_to_sig(s,stream);
        fprintf(stream," ***");
     }
#ifdef WIFCONTINUED
     else if (WIFCONTINUED(status))
     {
        fprintf(stream,"*** %i continued ***",cl->child_pid);
     }
#endif
     else
     {
        fprintf(stream,"has no associate signal");
     }
     return s;
}

void fd_to_name(int fd)
{
    fprintf(cl->out_stream,"fd(%i): ",fd);
    switch(fd)
    {
        case 0: fprintf(cl->out_stream,"stdin"); break;
        case 1: fprintf(cl->out_stream,"stdout"); break;
        case 2: fprintf(cl->out_stream,"stderr"); break;
        default: fprintf(cl->out_stream,"\'Unknown file description\'"); break;
    }
}

/* close input, output, error fd > /dev/null */
/* This function needs a rewrite ************************************************/
int close_ioe()
{
    int tfd;
    if((tfd=open(DEVNULL,O_RDWR))==-1)
    {
        return errno;
    }
    if (dup2(tfd, fileno(stdin))==-1)
    {
        return errno;
    }
    if (dup2(tfd, fileno(stdout))==-1)
    {
        return errno;
    }    
    if (dup2(tfd, fileno(stderr))==-1)
    {
        return errno;
    }
    if (close(tfd)!=0)
    {
        return errno;
    }    
    return 0;

/*
    if (fclose(stdin)!=0)
    {
        return errno;
    }
    if (fclose(stdout)!=0)
    {
        return errno;
    }
    if (fclose(stderr)!=0)
    {
        return errno;
    }
    if((tfd=open(DEVNULL,O_RDWR)==-1)
    {
        return errno;
    }
*/

}

#endif /* SIGNALS_H_ */
